@@ -1,24 +1,21 @@
 #
-# Copyright (c) CERN 2013-2015
+# Copyright (c) 2013-2022 CERN
 #
-# Copyright (c) Members of the EMI Collaboration. 2012-2013
-#    See  http://www.eu-emi.eu/partners for details on the copyright
+# Copyright (c) 2012-2013 Members of the EMI Collaboration
+#    See http://www.eu-emi.eu/partners for details on the copyright
 #    holders.
 #
-# This file is part of gfal2-util
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
 #
-# gfal2-util is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+#     http://www.apache.org/licenses/LICENSE-2.0
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
 #from __future__ import absolute_import # not available in python 2.4
 from __future__ import division
@@ -67,6 +64,7 @@ class GfalCommands(base.CommandBase):
             else:
                 break
 
+    @base.arg('-b', '--bytes', action='store_true', help="handle file contents as bytes (only in Python3)")
     @base.arg('file', action='store', nargs='+', type=base.surl, help="uri of the file to be displayed")
     def execute_cat(self):
         """
@@ -77,8 +75,12 @@ class GfalCommands(base.CommandBase):
             f = self.context.open(fname, 'r')
 
             while True:
-                b = f.read(65000)
-                sys.stdout.write(b)
+                if self.params.bytes and sys.version_info >= (3, 0):
+                    b = f.read_bytes(65000)
+                    sys.stdout.buffer.write(b)
+                else:
+                    b = f.read(65000)
+                    sys.stdout.write(b)
                 if not b:
                     break
 
@@ -160,3 +162,32 @@ class GfalCommands(base.CommandBase):
              self.parser.error('Mode must be an octal number (i.e. 0755)')
              return
          self.context.chmod(self.params.file, mode)
+
+    @base.arg('--issuer', action='store', type=str, help="token issuer URL")
+    @base.arg('--validity', action='store', type=int, default=60, help="token validity in minutes")
+    @base.arg('-w', '--write', dest='write_access', action='store_true', help="flag to request write access token")
+    @base.arg('path', action='store', type=base.surl, help="URI to request token for")
+    @base.arg('activities', action='store', nargs='*', type=str, help="activities for macaroon request")
+    def execute_token(self):
+        """
+        Retrieve a SE-issued token
+        """
+        if self.params.validity < 0:
+            sys.stderr.write("Validity must be a number >= 0\n")
+            return 1
+
+        if self.params.verbose:
+            if len(self.params.activities) > 0:
+                print("Will use user-provided activities")
+            else:
+                print("Will use default activities for {} access"
+                      .format(['read', 'write'][self.params.write_access]))
+
+        issuer = self.params.issuer if self.params.issuer is not None else ''
+        if len(self.params.activities) > 0:
+            token = self.context.token_retrieve(self.params.path, issuer, self.params.validity,
+                                                self.params.activities)
+        else:
+            token = self.context.token_retrieve(self.params.path, issuer, self.params.validity,
+                                                self.params.write_access)
+        sys.stdout.write(token + '\n')
